@@ -6,7 +6,7 @@
 use v5.8;
 
 package Parallel::WorkUnit;
-$Parallel::WorkUnit::VERSION = '1.006';
+$Parallel::WorkUnit::VERSION = '1.007';
 # ABSTRACT: Provide easy-to-use forking with ability to pass back data
 
 use strict;
@@ -135,30 +135,25 @@ sub waitone {
     my $sp = $self->_subprocs();
     if ( !keys(%$sp) ) { return undef; }
 
-    if ($do_thread) {
-        my $child = $self->_queue()->dequeue(1);
+    my $s = IO::Select->new();
+    foreach ( keys(%$sp) ) { $s->add( $sp->{$_}{fh} ); }
 
-        my $thr = $self->_subprocs()->{$child}{thread};
-        $self->_read_result($child);
+    my @ready = $s->can_read();
 
-        $thr->join();
-    } else {
-        my $s = IO::Select->new();
-        foreach ( keys(%$sp) ) { $s->add( $sp->{$_}{fh} ); }
-
-        my @ready = $s->can_read();
-
-        foreach my $fh (@ready) {
-            foreach my $child ( keys(%$sp) ) {
-                if ( defined($fh->fileno())) {
-                    if ( $fh->fileno() == $sp->{$child}{fh}->fileno() ) {
-                        my $thr = $self->_subprocs()->{$child}{thread};
-                        $self->_read_result($child);
+    foreach my $fh (@ready) {
+        foreach my $child ( keys(%$sp) ) {
+            if ( defined($fh->fileno())) {
+                if ( $fh->fileno() == $sp->{$child}{fh}->fileno() ) {
+                    my $thr = $self->_subprocs()->{$child}{thread};
+                    $self->_read_result($child);
 
+                    if ($do_thread) {
+                        $thr->join();
+                    } else {
                         waitpid($child, 0);
-
-                        return 1;  # We don't want to read more than one!
                     }
+
+                    return 1;  # We don't want to read more than one!
                 }
             }
         }
@@ -302,7 +297,7 @@ Parallel::WorkUnit - Provide easy-to-use forking with ability to pass back data
 
 =head1 VERSION
 
-version 1.006
+version 1.007
 
 =head1 SYNOPSIS
 
